import { Injectable } from '@nestjs/common';
import { PrismaService } from '../prisma/prisma.service';
import * as bcrypt from 'bcryptjs';
import { User } from '@prisma/client';  
import { CreateUserDto } from './dto/CreateUserDto ';

@Injectable()
export class UserService {
  constructor(private prisma: PrismaService) {}

  async createUser(body: CreateUserDto): Promise<User> {
    const hashedPassword = await bcrypt.hash(body.password, 10);

    const user = await this.prisma.user.create({
      data: {
        name: body.name,
        document: body.document,
        email: body.email,
        password: hashedPassword,
      },
    });
  
    await this.prisma.userRole.create({
      data: {
        userId: user.id,
        roleId: 2,
      },
    });
  
    return user;
  }

  async findOneByDocument(document: string): Promise<User | null> {
    return this.prisma.user.findUnique({  
      where: {
        document, 
      },
    });
  }
  
  async findOneById(id: number): Promise<User | null> {
    return this.prisma.user.findUnique({  
      where: {
        id, 
      },
    });
  }

  async findUserAndRoleByDocument(document: string) {
    const user = await this.prisma.user.findUnique({
      where: { document },
      include: {
        roles: {
          select: {
            role: {  // Acessa a relação de roles e seleciona o campo 'name'
              select: {
                name: true,  // Seleciona somente o nome das roles
              },
            },
          },
        },
      },
    });
    
  
    if (user) {
      // Extraímos apenas os nomes das roles associadas ao usuário
      const roleNames = user.roles ? user.roles.map(role => role.role.name) : [];
      return {
        ...user,
        roles: roleNames,  // Retorna um array de nomes das roles
      };
    }
  
    return null;  // Retorna null se o usuário não for encontrado
  }


  async findUserAndRoleByEmail(email: string) {
    console.log(email)
    const user = await this.prisma.user.findUnique({
      where: { email: email },
      include: {
        roles: {
          select: {
            role: { 
              select: {
                name: true,  
              },
            },
          },
        },
      },
    });
    
  
    if (user) {
      const roleNames = user.roles ? user.roles.map(role => role.role.name) : [];
      return {
        ...user,
        roles: roleNames,
      };
    }
  
    return null;  // Retorna null se o usuário não for encontrado
  }
  


  async validatePassword(document: string, password: string): Promise<boolean> {
    const user = await this.findOneByDocument(document);

    if (user) {
      const isPasswordValid = await bcrypt.compare(password, user.password);
      return isPasswordValid;
    }

    return false;  
  }
}
